# Movable Type (r) (C) 2007 Six Apart, Ltd. All Rights Reserved.
# This code cannot be redistributed without permission from www.sixapart.com.
# For more information, consult your Movable Type license.
#
# $Id$

# Original Copyright (c) 2005-2007 Arvind Satyanarayan

package CustomFields::Field;

use strict;
use base qw( MT::Object );

__PACKAGE__->install_properties({
    column_defs => {
        'id' => 'integer not null auto_increment',
        'blog_id' => 'integer',
        'name' => 'string(255) not null',
        'description' => 'text',
        'obj_type' => 'string(50) not null',
        'type' => 'string(50) not null',
        'tag' => 'string(255) not null',
        'default' => 'text',
        'options' => 'text',
        'required' => 'boolean',
        'basename' => 'string(255)',
    },
    indexes => {
        blog_id => 1,
        name => 1,
        obj_type => 1,
        type => 1,
        basename => 1
    },
    primary_key => 'id',
    datasource => 'field',
});

sub class_label {
    return MT->translate("Field");
}

sub class_label_plural {
    return MT->translate("Fields");
}

sub save {
    my $field = shift;

    ## If there's no basename specified, create a unique basename.
    if (!defined($field->basename) || ($field->basename eq '')) {
        require CustomFields::Util;
        my $name = CustomFields::Util::make_unique_field_basename($field);
        $field->basename($name);
    }

    my $id = $field->id;
    my $orig_basename;

    if ($id) {
        if ($field->{changed_cols}{basename}) {
            my $orig_obj = __PACKAGE__->load($id);
            $orig_basename = $orig_obj->basename;
        }
    }

    my $res = $field->SUPER::save(@_);
    if ($res) {
        if (! $id) { # new meta field
            # install it!
            $field->add_meta();
        }
        if ( defined $orig_basename ) {
            # update existing meta records to use new basename
            my $ppkg = MT->model($field->obj_type);
            if ($ppkg) {
                if (my $mpkg = $ppkg->meta_pkg) {
                    my $driver = $mpkg->driver;
                    my $type_col = $driver->dbd->db_column_name($mpkg->datasource, 'type');
                    my $dbh = $driver->r_handle;
                    my $basename = $dbh->quote($field->basename);
                    $orig_basename = $dbh->quote($orig_basename);

                    $driver->sql('update ' . $mpkg->table_name
                        . qq{ set $type_col='field.$basename'}
                        . qq{ where $type_col='field.$orig_basename'});

                    $driver->clear_cache if $driver->can('clear_cache');
                }
            }
        }
    }

    return $res;
}

sub parents {
    my $obj = shift;
    {
        blog_id => { class => MT->model('blog'), optional => 1 },
    };
}

sub add_meta {
    my $field = shift;

    my $ppkg = MT->model($field->obj_type);
    return 0 unless $ppkg;

    my $types = MT->registry("customfield_types");

    my $fields = {};
    my $cf_type = $types->{$field->type}
        or return 0;

    my $type = $cf_type->{column_def} || 'vblob';
    $ppkg->install_meta({ column_defs => {
        'field.' . $field->basename => $type
    }});
    return 1;
}

1;
